// Multiple Lagged Fibonacci RNGs
// See Knuth, The Art of Computer Programming, v. 2, p. 26ff
#ifndef LFRNG_H
#define LFRNG_H

#include <stdexcept>

namespace LFRNG {

class rng {
    unsigned k, l;
    unsigned imk, iml;
    unsigned *hstr;

    struct pair {
        unsigned k, l;
        pair(unsigned i, unsigned j) : k(i), l(j) {}
    };

    const static unsigned n_lags = 3;
    const static pair lags[n_lags];

    void build(unsigned n);
    void random_init();
    void array_init(const unsigned *a);
    void copy_in(const rng& g);
protected:
    rng(const rng& g) { copy_in(g); }
    rng& operator= (const rng& g);
    unsigned draw();
public:
    explicit rng(unsigned n = 98) { build(n); random_init(); }
    rng(unsigned n, const unsigned *a) {
        build(n);
        if (n==k) array_init(a); 
        else throw std::invalid_argument("unsupported length");
    }
    ~rng();
    static unsigned max_len() { return lags[n_lags-1].k; }
    unsigned len() const { return k; }
    unsigned operator() () { return draw(); }
};

} //namespace LFRNG

#endif
