#ifndef FRNG_H
#define FRNG_H

#include <limits>
#include <cmath>
#include "lfrng.h"

namespace FPRNG {

class frng {
    LFRNG::rng intgen;
protected:
    double draw() {
             return double(intgen())/std::numeric_limits<unsigned>::max(); }
public:
    explicit frng(unsigned n = 98) : intgen(n) {}
    frng(unsigned n, const unsigned *a): intgen(n, a) {}
    unsigned len() { return intgen.len(); }
    static unsigned max_len() { return LFRNG::rng::max_len(); }
    virtual double operator() () = 0;
};

class furng : public frng {
public:
    explicit furng(unsigned n = 98) : frng(n) {}
    furng(unsigned n, const unsigned *a): frng(n, a) {}
    virtual double operator() () { return frng::draw(); }
};

class fsurng : public frng {
    double offset, scale;
public:
    fsurng(double o, double s, unsigned n = 98) : offset(o), scale(s), frng(n) {}
    fsurng(unsigned n, const unsigned *a): frng(n, a) {}
    virtual double operator() () { return frng::draw()*scale + offset; }
};

class ferng : public frng {
public:
    explicit ferng(unsigned n = 98) : frng(n) {}
    ferng(unsigned n, const unsigned *a): frng(n, a) {}
    virtual double operator() ();
};

class fnrng : public frng {
    const static double pi2 = 2.0*3.1415926535897932384626433832795;
    double ndr;
    bool cached;
public:
    explicit fnrng(unsigned n = 98) : cached(false), frng(n) {}
    fnrng(unsigned n, const unsigned *a): cached(false), frng(n, a) {}
    virtual double operator() ();
};

} // namespace FPRNG

#endif
