#include <cstdlib>
#include <cerrno>
#include <cstring>
#include <iostream>
#include <stdexcept>
#include <cstdio>
#include "lfrng.h"

using namespace LFRNG;

const rng::pair rng::lags[rng::n_lags] = {rng::pair(55,24),
                                                rng::pair(73,31),
                                                rng::pair(98,27)};


void rng::build(unsigned n) {
    int i;

    for(i = 0; i < n_lags; ++i) {
        l = lags[i].l;
        k = lags[i].k;
        if (n <= k) break;
    }
    if (n > k) errno = EDOM;

    hstr = new unsigned[k];
}


void rng::random_init() {
    int i;

    for(i=0; i<k; ++i)
        hstr[k-i-1] = rand();

    imk = k-1;
    iml = l-1;
}


void rng::array_init(const unsigned *a) {
    memcpy(hstr, a, k*sizeof(unsigned));
    imk = k-1;
    iml = l-1;
}

void rng::copy_in(const rng& g) {
    k = g.k;
    l = g.l;
    hstr = new unsigned[k];
    memcpy(hstr, g.hstr, k*sizeof(unsigned));
    imk = g.imk;
    iml = g.iml;
}

rng& rng::operator= (const rng& g) {
    if (this != &g) {
        delete[] hstr;
        copy_in(g);
    }
    return *this;
}


rng::~rng() {
    delete[] hstr;
}


unsigned rng::draw() {
    unsigned r;

    r = hstr[imk] + hstr[iml];
    hstr[imk] = r;
    if (imk-- == 0) imk = k-1;
    if (iml-- == 0) iml = k-1;

    return r;
}

namespace LFRNG {

std::ostream& operator<< (std::ostream& s, const rng& g) {
    int i;

    s << g.k << ' ' << g.l << std::endl;
    s << g.imk << ' ' << g.iml << std::endl;
    for(i = 0; i<g.k; ++i)
        s << g.hstr[i] << ' ';
    s << std::endl;

    return s;
}

std::istream& operator>> (std::istream& s, rng& g) {
    unsigned k, l, imk, iml, fcs;
    unsigned *hstr;
    k = l = 0;

    s >> k >> l;
    if (k != g.k || l != g.l) {
        s.clear(std::ios_base::failbit);
        return s;
    } else {
        hstr = new unsigned[k];
        s >> imk >> iml;
        for(int i = 0; i<k; ++i)
            s >> hstr[i];
    }
    if (s) {
        g.k = k;
        g.l = l;
        g.imk = imk;
        g.iml = iml;
        memcpy(g.hstr, hstr, k*sizeof(unsigned));
    }
    delete[] hstr;
    return s;
}

std::ostream& operator<< (std::ostream& s, const nofirst& g) {

    return s << static_cast<const rng&>(g) << g.takeit << ' ' << g.first << std::endl;
}

std::istream& operator>> (std::istream& s, nofirst& g) {
    nofirst temp(g);

    s >> static_cast<rng&>(temp);
    if (s)
        s >> temp.takeit >> temp.first;
    if (s)
        g = temp;
    return s;
}

} //namespace LFRNG
