#ifndef POSITION_H
#define POSITION_H

#include <cmath>

struct position {
    double x, y, z;
    
    position() {}
    position(double a, double b, double c) : x(a), y(b), z(c) {}
    
    position operator+() const { return *this; }
    position operator-() const { return position(-x, -y, -z); }
    
    position& operator+= (position r) { x += r.x; y += r.y; z += r.z; return *this; }
    position& operator-= (position r) { x -= r.x; y -= r.y; z -= r.z; return *this; }
    position& operator*= (double s) { x *= s; y *= s; z *= s; return *this; }
    position& operator/= (double s) { x /= s; y /= s; z /= s; return *this; }

    position operator+ (position r) const;
    position operator- (position r) const;
    double operator* (position r) const;
    position operator* (double s) const;
    friend position operator* (double s, position r);
    position operator/ (double s) const;
    bool operator== (position r) const;
    bool operator!= (position r) const;
    
    friend double abs(position r);
};

inline position position::operator+ (position r) const {
    position temp(*this);
    temp += r;
    return temp;
}

inline position position::operator- (position r) const {
    position temp(*this);
    temp -= r;
    return temp;
}

inline double position::operator* (position r) const {
    return x*r.x + y*r.y + z*r.z;
}

inline position position::operator* (double s) const {
    position temp(*this);
    temp *= s;
    return temp;
}
    
inline position operator* (double s, position r) {
    r *= s;
    return r;
}
    
inline position position::operator/ (double s) const {
    position temp(*this);
    temp /= s;
    return temp;
}

inline bool position::operator== (position r) const {
    return x==r.x && y==r.y && z==r.z;
}

inline bool position::operator!= (position r) const {
    return !(*this==r);
}

inline double abs(position r) {
    return sqrt(r*r);
}

#endif
