#ifndef VECT_H
#define VECT_H

#include <iostream>
#include <cmath>

template<class T> struct vect {
    T x, y, z;
  protected:    
    vect() {}
    vect(T a, T b, T c) : x(a), y(b), z(c) {}
    
    // mathematical
    vect operator+() const { return *this; }
    vect operator-() const { return vect(-x, -y, -z); }
    
    vect operator+ (vect r) const;
    vect operator- (vect r) const;
    T operator* (vect r) const;
    vect operator* (T s) const;
    vect operator/ (T s) const;

    // assignment
    vect& operator+= (vect r) { x += r.x; y += r.y; z += r.z; return *this; }
    vect& operator-= (vect r) { x -= r.x; y -= r.y; z -= r.z; return *this; }
    vect& operator*= (T s) { x *= s; y *= s; z *= s; return *this; }
    vect& operator/= (T s) { x /= s; y /= s; z /= s; return *this; }

    // logical
    bool operator== (vect r) const;
    bool operator!= (vect r) const;
    
    // must use a different template argument name
    template<class F> friend vect<F> operator* (F s, vect<F> r);
    template<class F> friend T abs(vect<F> r);

  public:
    template<class F> friend std::ostream & operator<<(std::ostream &s, const vect<F> &r);
    
};

template<class T> inline vect<T> vect<T>::operator+ (vect<T> r) const {
    vect<T> temp(*this);
    temp += r;
    return temp;
}

template<class T> inline vect<T> vect<T>::operator- (vect<T> r) const {
    vect<T> temp(*this);
    temp -= r;
    return temp;
}

template<class T> inline T vect<T>::operator* (vect<T> r) const {
    return x*r.x + y*r.y + z*r.z;
}

template<class T> inline vect<T> vect<T>::operator* (T s) const {
    vect<T> temp(*this);
    temp *= s;
    return temp;
}
    
template<class T> inline vect<T> operator* (T s, vect<T> r) {
    r *= s;
    return r;
}
    
template<class T> inline vect<T> vect<T>::operator/ (T s) const {
    vect<T> temp(*this);
    temp /= s;
    return temp;
}

template<class T> inline bool vect<T>::operator== (vect<T> r) const {
    return x==r.x && y==r.y && z==r.z;
}

template<class T> inline bool vect<T>::operator!= (vect<T> r) const {
    return !(*this==r);
}

template<class T> inline T abs(vect<T> r) {
    return sqrt(r*r);
}

// stream
template<class F> std::ostream & operator<<(std::ostream &s, const vect<F> &r) 
{
  s << "( " << r.x << ", " << r.y << ", " << r.z << " )" << std::endl;
}

#endif
