#ifndef POLINOMIAL_H
#define POLINOMIAL_H

#include <cstring>

template<int N, class T=double> class polynomial {
    T coeff[N+1];
    
    template<class U, int M, int C> struct horner {
        static U eval(const U *c, U x) {
            return horner<U,M,C-1>::eval(c, x)*x + c[M-C];
        }
    };

    template<class U, int M> struct horner<U,M,0> {
        static U eval(const U *c, U x) {
            return c[M];
        }
    };

public:
    polynomial() { 
        for (int i = 0; i<N+1; ++i)
            coeff[i] = 0.0;
    }
    polynomial(const T c[]) {
        memcpy(coeff, c, (N+1)*sizeof(T));
    }
    
    T& operator[] (int i) { return coeff[i]; }

    T operator() (T x) const { return horner<T,N,N>::eval(coeff, x); }
};

#endif